@@ -14,11 +14,10 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
-#include <glib.h>
 
 /* Check the Unix98 goodie */
 #ifndef socklen_t
-   #define socklen_t guint32
+   #define socklen_t uint32_t
 #endif
 
 /* Some systems may not have this one... */
